@@ -94,6 +94,12 @@ int cm_dnsEnabled = 1;
 
 char cm_NetBiosName[32];
 
+extern initUpperCaseTable();
+void afsd_initUpperCaseTable() 
+{
+	initUpperCaseTable();
+}
+
 void
 afsi_start()
 {
@@ -163,11 +169,12 @@ void afsd_ForceTrace(BOOL flush)
 {
 	HANDLE handle;
 	int len;
-	char buf[100];
+	char buf[256];
 
-	if (!logReady) return;
+	if (!logReady) 
+        return;
 
-	len = GetTempPath(99, buf);
+	len = GetTempPath(sizeof(buf)-10, buf);
 	strcpy(&buf[len], "/afsd.log");
 	handle = CreateFile(buf, GENERIC_WRITE, FILE_SHARE_READ,
 			    NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
@@ -210,11 +217,14 @@ int afsd_InitCM(char **reasonP)
 
 	WSAStartup(0x0101, &WSAjunk);
 
+    afsd_initUpperCaseTable();
+
 	/* setup osidebug server at RPC slot 1000 */
 	osi_LongToUID(1000, &debugID);
 	code = osi_InitDebug(&debugID);
 	afsi_log("osi_InitDebug code %d", code);
-//	osi_LockTypeSetDefault("stat");	/* comment this out for speed *
+
+    //	osi_LockTypeSetDefault("stat");	/* comment this out for speed *
 	if (code != 0) {
 		*reasonP = "unknown error";
 		return -1;
@@ -242,6 +252,24 @@ int afsd_InitCM(char **reasonP)
 		osi_panic(buf, __FILE__, __LINE__);
 	}
 
+	dummyLen = sizeof(traceBufSize);
+	code = RegQueryValueEx(parmKey, "TraceBufferSize", NULL, NULL,
+				(BYTE *) &traceBufSize, &dummyLen);
+	if (code == ERROR_SUCCESS)
+		afsi_log("Trace Buffer size %d", traceBufSize);
+	else {
+		traceBufSize = CM_CONFIGDEFAULT_TRACEBUFSIZE;
+		afsi_log("Default trace buffer size %d", traceBufSize);
+	}
+
+	/* setup and enable debug log */
+	afsd_logp = osi_LogCreate("afsd", traceBufSize);
+	afsi_log("osi_LogCreate log addr %x", (int)afsd_logp);
+    osi_LogEnable(afsd_logp);
+	logReady = 1;
+
+    osi_Log0(afsd_logp, "Log init");
+
 	dummyLen = sizeof(cacheSize);
 	code = RegQueryValueEx(parmKey, "CacheSize", NULL, NULL,
 				(BYTE *) &cacheSize, &dummyLen);
@@ -308,7 +336,7 @@ int afsd_InitCM(char **reasonP)
 		ltt = 1;
 		afsi_log("Logoff token transfer on by default");
 	}
-        smb_LogoffTokenTransfer = ltt;
+    smb_LogoffTokenTransfer = ltt;
 
 	if (ltt) {
 		dummyLen = sizeof(ltto);
@@ -321,8 +349,10 @@ int afsd_InitCM(char **reasonP)
 			ltto = 10;
 			afsi_log("Default logoff token transfer timeout 10 seconds");
 		}
-	}
-        smb_LogoffTransferTimeout = ltto;
+	} else {
+        ltto = 0;
+    }
+    smb_LogoffTransferTimeout = ltto;
 
 	dummyLen = sizeof(cm_rootVolumeName);
 	code = RegQueryValueEx(parmKey, "RootVolume", NULL, NULL,
@@ -380,16 +410,6 @@ int afsd_InitCM(char **reasonP)
 		/* Don't log */
 	}
 
-	dummyLen = sizeof(traceBufSize);
-	code = RegQueryValueEx(parmKey, "TraceBufferSize", NULL, NULL,
-				(BYTE *) &traceBufSize, &dummyLen);
-	if (code == ERROR_SUCCESS)
-		afsi_log("Trace Buffer size %d", traceBufSize);
-	else {
-		traceBufSize = CM_CONFIGDEFAULT_TRACEBUFSIZE;
-		afsi_log("Default trace buffer size %d", traceBufSize);
-	}
-
 	dummyLen = sizeof(cm_sysName);
 	code = RegQueryValueEx(parmKey, "SysName", NULL, NULL,
 				cm_sysName, &dummyLen);
@@ -527,14 +547,6 @@ int afsd_InitCM(char **reasonP)
 	/* turn from 1024 byte units into memory blocks */
     cacheBlocks = (cacheSize * 1024) / buf_bufferSize;
         
-	/* setup and enable debug log */
-	afsd_logp = osi_LogCreate("afsd", traceBufSize);
-	afsi_log("osi_LogCreate log addr %x", (int)afsd_logp);
-    osi_LogEnable(afsd_logp);
-	logReady = 1;
-
-    osi_Log0(afsd_logp, "Log init");
-
 	/* get network related info */
 	cm_noIPAddr = CM_MAXINTERFACE_ADDR;
 	code = syscfg_GetIFInfo(&cm_noIPAddr,
@@ -571,6 +583,27 @@ int afsd_InitCM(char **reasonP)
         afsi_log("rx_SetMaxMTU %d successful", rx_mtu);
     }
 
+    /* Open Microsoft Firewall to allow in port 7001 */
+    {
+        HKEY hk;
+        DWORD dwDisp;
+        TCHAR* value = TEXT("7001:UDP:*:Enabled:AFS Cache Manager Callback");
+        if (RegCreateKeyEx (HKEY_LOCAL_MACHINE, 
+                            "SYSTEM\\CurrentControlSet\\Services\\SharedAccess\\Parameters\\FirewallPolicy\\DomainProfile\\GloballyOpenP", 
+                            0, TEXT("container"), 0, KEY_SET_VALUE, NULL, &hk, &dwDisp) == ERROR_SUCCESS)
+        {
+            RegSetValueEx (hk, TEXT("7001:UDP"), 0, REG_SZ, (PBYTE)value, sizeof(TCHAR) * (1+lstrlen(value)));
+            RegCloseKey (hk);
+        }
+        if (RegCreateKeyEx (HKEY_LOCAL_MACHINE, 
+                            "SYSTEM\\CurrentControlSet\\Services\\SharedAccess\\Parameters\\FirewallPolicy\\StandardProfile\\GloballyOpenP", 
+                            0, TEXT("container"), 0, KEY_SET_VALUE, NULL, &hk, &dwDisp) == ERROR_SUCCESS)
+        {
+            RegSetValueEx (hk, TEXT("7001:UDP"), 0, REG_SZ, (PBYTE)value, sizeof(TCHAR) * (1+lstrlen(value)));
+            RegCloseKey (hk);
+        }
+    }
+
 	/* initialize RX, and tell it to listen to port 7001, which is used for
      * callback RPC messages.
      */
