@@ -17,7 +17,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/LINUX/osi_groups.c,v 1.18 2003/07/15 23:14:24 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_groups.c,v 1.22 2004/04/21 02:20:23 shadow Exp $");
 
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
@@ -26,29 +26,99 @@ RCSID
 #include "h/smp_lock.h"
 #endif
 
-static int afs_getgroups(cred_t * cr, gid_t * groups);
-static int afs_setgroups(cred_t ** cr, int ngroups, gid_t * gidset,
-			 int change_parent);
+#if defined(AFS_LINUX26_ENV)
+static int
+afs_setgroups(cred_t **cr, struct group_info *group_info, int change_parent)
+{
+    struct group_info *old_info;
 
+    AFS_STATCNT(afs_setgroups);
+
+    old_info = (*cr)->cr_group_info;
+    get_group_info(group_info);
+    (*cr)->cr_group_info = group_info;
+    put_group_info(old_info);
+
+    crset(*cr);
+
+    if (change_parent) {
+	old_info = current->parent->group_info;
+	get_group_info(group_info);
+	current->parent->group_info = group_info;
+	put_group_info(old_info);
+    }
+
+    return (0);
+}
+#else
+static int
+afs_setgroups(cred_t **cr, int ngroups, gid_t * gidset, int change_parent)
+{
+    int ngrps;
+    int i;
+    gid_t *gp;
+
+    AFS_STATCNT(afs_setgroups);
+
+    if (ngroups > NGROUPS)
+	return EINVAL;
+
+    gp = (*cr)->cr_groups;
+    if (ngroups < NGROUPS)
+	gp[ngroups] = (gid_t) NOGROUP;
+
+    for (i = ngroups; i > 0; i--) {
+	*gp++ = *gidset++;
+    }
+
+    (*cr)->cr_ngroups = ngroups;
+    crset(*cr);
+    return (0);
+}
+#endif
+
+#if defined(AFS_LINUX26_ENV)
+static struct group_info *
+afs_getgroups(cred_t * cr)
+{
+    AFS_STATCNT(afs_getgroups);
+
+    get_group_info(cr->cr_group_info);
+    return cr->cr_group_info;
+}
+#else
+/* Returns number of groups. And we trust groups to be large enough to
+ * hold all the groups.
+ */
+static int
+afs_getgroups(cred_t *cr, gid_t *groups)
+{
+    int i;
+    AFS_STATCNT(afs_getgroups);
+
+    gid_t *gp = cr->cr_groups;
+    int n = cr->cr_ngroups;
+
+    for (i = 0; (i < n) && (*gp != (gid_t) NOGROUP); i++)
+	*groups++ = *gp++;
+    return i;
+}
+#endif
+
+#if !defined(AFS_LINUX26_ENV)
 /* Only propogate the PAG to the parent process. Unix's propogate to 
  * all processes sharing the cred.
  */
 int
 set_pag_in_parent(int pag, int g0, int g1)
 {
+    int i;
 #ifdef STRUCT_TASK_STRUCT_HAS_PARENT
     gid_t *gp = current->parent->groups;
+    int ngroups = current->parent->ngroups;
 #else
     gid_t *gp = current->p_pptr->groups;
-#endif
-    int ngroups;
-    int i;
-
-
-#ifdef STRUCT_TASK_STRUCT_HAS_PARENT
-    ngroups = current->parent->ngroups;
-#else
-    ngroups = current->p_pptr->ngroups;
+    int ngroups = current->p_pptr->ngroups;
 #endif
 
     if ((ngroups < 2) || (afs_get_pag_from_groups(gp[0], gp[1]) == NOPAG)) {
@@ -73,11 +143,45 @@ set_pag_in_parent(int pag, int g0, int g1)
 #endif
     return 0;
 }
+#endif
 
 int
 setpag(cred_t ** cr, afs_uint32 pagvalue, afs_uint32 * newpag,
        int change_parent)
 {
+#if defined(AFS_LINUX26_ENV)
+    struct group_info *group_info;
+    gid_t g0, g1;
+
+    AFS_STATCNT(setpag);
+
+    group_info = afs_getgroups(*cr);
+    g0 = GROUP_AT(group_info, 0);
+    g1 = GROUP_AT(group_info, 1);
+
+    if (afs_get_pag_from_groups(g0, g1) == NOPAG) {
+	/* We will have to make sure group_info is big enough for pag */
+	struct group_info *tmp;
+	int i;
+	
+	tmp = groups_alloc(group_info->ngroups + 2);
+	for (i = 0; i < group_info->ngroups; ++i)
+		GROUP_AT(tmp, i + 2) = GROUP_AT(group_info, i);
+	put_group_info(group_info);
+	group_info = tmp;
+    }
+
+    *newpag = (pagvalue == -1 ? genpag() : pagvalue);
+    afs_get_groups_from_pag(*newpag, &g0, &g1);
+    GROUP_AT(group_info, 0) = g0;
+    GROUP_AT(group_info, 1) = g1;
+
+    afs_setgroups(cr, group_info, change_parent);
+
+    put_group_info(group_info);
+
+    return 0;
+#else
     gid_t *gidset;
     afs_int32 ngroups, code = 0;
     int j;
@@ -111,6 +215,7 @@ setpag(cred_t ** cr, afs_uint32 pagvalue, afs_uint32 * newpag,
 
     osi_Free((char *)gidset, NGROUPS * sizeof(int));
     return code;
+#endif
 }
 
 
@@ -183,27 +288,26 @@ afs_xsetgroups32(int gidsetsize, gid_t * grouplist)
 }
 #endif
 
-#if defined(AFS_SPARC64_LINUX20_ENV) || defined(AFS_AMD64_LINUX20_ENV)
+#if defined(AFS_PPC64_LINUX20_ENV)
 /* Intercept the uid16 system call as used by 32bit programs. */
-extern long (*sys32_setgroupsp) (int gidsetsize, old_gid_t * grouplist);
-asmlinkage long
-afs32_xsetgroups(int gidsetsize, old_gid_t * grouplist)
+extern long (*sys32_setgroupsp)(int gidsetsize, gid_t *grouplist);
+asmlinkage long afs32_xsetgroups(int gidsetsize, gid_t *grouplist)
 {
     long code;
     cred_t *cr = crref();
     afs_uint32 junk;
     int old_pag;
-
+    
     lock_kernel();
     old_pag = PagInCred(cr);
     crfree(cr);
     unlock_kernel();
-
-    code = (*sys32_setgroupsp) (gidsetsize, grouplist);
+    
+    code = (*sys32_setgroupsp)(gidsetsize, grouplist);
     if (code) {
 	return code;
     }
-
+    
     lock_kernel();
     cr = crref();
     if (old_pag != NOPAG && PagInCred(cr) == NOPAG) {
@@ -212,7 +316,42 @@ afs32_xsetgroups(int gidsetsize, old_gid_t * grouplist)
     }
     crfree(cr);
     unlock_kernel();
+    
+    /* Linux syscall ABI returns errno as negative */
+    return (-code);
+}
+#endif
 
+#if defined(AFS_SPARC64_LINUX20_ENV) || defined(AFS_AMD64_LINUX20_ENV)
+/* Intercept the uid16 system call as used by 32bit programs. */
+extern long (*sys32_setgroupsp) (int gidsetsize, u16 * grouplist);
+asmlinkage long
+afs32_xsetgroups(int gidsetsize, u16 * grouplist)
+{
+    long code;
+    cred_t *cr = crref();
+    afs_uint32 junk;
+    int old_pag;
+    
+    lock_kernel();
+    old_pag = PagInCred(cr);
+    crfree(cr);
+    unlock_kernel();
+    
+    code = (*sys32_setgroupsp) (gidsetsize, grouplist);
+    if (code) {
+	return code;
+    }
+    
+    lock_kernel();
+    cr = crref();
+    if (old_pag != NOPAG && PagInCred(cr) == NOPAG) {
+	/* re-install old pag if there's room. */
+	code = setpag(&cr, old_pag, &junk, 0);
+    }
+    crfree(cr);
+    unlock_kernel();
+    
     /* Linux syscall ABI returns errno as negative */
     return (-code);
 }
@@ -253,44 +392,3 @@ afs32_xsetgroups32(int gidsetsize, gid_t * grouplist)
 #endif
 #endif
 
-static int
-afs_setgroups(cred_t ** cr, int ngroups, gid_t * gidset, int change_parent)
-{
-    int ngrps;
-    int i;
-    gid_t *gp;
-
-    AFS_STATCNT(afs_setgroups);
-
-    if (ngroups > NGROUPS)
-	return EINVAL;
-
-    gp = (*cr)->cr_groups;
-    if (ngroups < NGROUPS)
-	gp[ngroups] = (gid_t) NOGROUP;
-
-    for (i = ngroups; i > 0; i--) {
-	*gp++ = *gidset++;
-    }
-
-    (*cr)->cr_ngroups = ngroups;
-    crset(*cr);
-    return (0);
-}
-
-/* Returns number of groups. And we trust groups to be large enough to
- * hold all the groups.
- */
-static int
-afs_getgroups(cred_t * cr, gid_t * groups)
-{
-    int i;
-    gid_t *gp = cr->cr_groups;
-    int n = cr->cr_ngroups;
-    AFS_STATCNT(afs_getgroups);
-
-    for (i = 0; (i < n) && (*gp != (gid_t) NOGROUP); i++) {
-	*groups++ = *gp++;
-    }
-    return i;
-}
