@@ -71,6 +71,7 @@ extern int afs_shuttingdown;
 #define	NFENTRIES	256	/* hash table size for disk volume table */
 #define	VCSIZE	       1024	/* stat cache hash table size */
 #define	DCSIZE		512	/* disk cache hash table size */
+#define CBRSIZE		512	/* call back returns hash table size */
 #define	PIGGYSIZE	1350	/* max piggyback size */
 #define	MAXVOLS		128	/* max vols we can store */
 #define	MAXSYSNAME	128	/* max sysname (i.e. @sys) size */
@@ -191,7 +192,12 @@ struct vrequest {
  * callbacks.  Sent asynchronously when we run a little low on free dudes.
  */
 struct afs_cbr {
+    struct afs_cbr **pprev;
     struct afs_cbr *next;
+
+    struct afs_cbr **hash_pprev;
+    struct afs_cbr *hash_next;
+
     struct AFSFid fid;
 };
 
@@ -551,7 +557,7 @@ struct SimpleLocks {
 #define vrefCount   v.v_count
 #endif /* AFS_XBSD_ENV */
 
-#ifdef AFS_LINUX24_ENV
+#if defined(AFS_LINUX24_ENV)
 #define VREFCOUNT(v)		atomic_read(&((vnode_t *) v)->v_count)
 #define VREFCOUNT_SET(v, c)	atomic_set(&((vnode_t *) v)->v_count, c)
 #define VREFCOUNT_DEC(v)	atomic_dec(&((vnode_t *) v)->v_count)
@@ -926,6 +932,7 @@ struct fcache {
 /* kept in memory */
 struct dcache {
     struct afs_q lruq;		/* Free queue for in-memory images */
+    struct afs_q dirty;		/* Queue of dirty entries that need written */
     afs_rwlock_t lock;		/* Protects validPos, some f */
     afs_rwlock_t tlock;		/* Atomizes updates to refCount */
     afs_rwlock_t mflock;	/* Atomizes accesses/updates to mflags */
